/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.system.user.feign;


import org.springblade.core.launch.constant.AppConstant;
import org.springblade.core.tool.api.R;
import org.springblade.system.user.entity.User;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * User Search Feign接口类
 *
 * @author Chill
 */
@FeignClient(
        value = AppConstant.APPLICATION_USER_NAME
)
public interface IUserSearchClient {

    String API_PREFIX = "/client";
    String LIST_BY_USER = API_PREFIX + "/user/list-by-user";
    String LIST_BY_DEPT = API_PREFIX + "/user/list-by-dept";
    String LIST_BY_POST = API_PREFIX + "/user/list-by-post";
    String LIST_BY_ROLE = API_PREFIX + "/user/list-by-role";

    String MAP_BY_USER = API_PREFIX + "/user/map-by-role";
    String GET_NAME_BY_USER_ID = API_PREFIX + "/user/get-user-name-and-post-by-user-id";
    String GET_USER_ID_BY_JOB_NO = API_PREFIX + "/user/get-user-id-by-job-no";

    /**
     * 根据用户ID查询用户列表
     *
     * @param userId 用户ID
     * @return 用户列表
     */
    @GetMapping(LIST_BY_USER)
    R<List<User>> listByUser(@RequestParam("userId") String userId);

    /**
     * 根据部门ID查询用户列表
     *
     * @param deptId 部门ID
     * @return 用户列表
     */
    @GetMapping(LIST_BY_DEPT)
    R<List<User>> listByDept(@RequestParam("deptId") String deptId);

    /**
     * 根据岗位ID查询用户列表
     *
     * @param postId 岗位ID
     * @return 用户列表
     */
    @GetMapping(LIST_BY_POST)
    R<List<User>> listByPost(@RequestParam("postId") String postId);

    /**
     * 根据角色ID查询用户列表
     *
     * @param roleId 角色ID
     * @return 用户列表
     */
    @GetMapping(LIST_BY_ROLE)
    R<List<User>> listByRole(@RequestParam("roleId") String roleId);

    /**
     * 根据用户id查询用户名称
     *
     * @param userIdList 用户id
     * @return MAP<用户id, 用户名称>
     */
    @GetMapping(MAP_BY_USER)
    R<Map<Long, String>> mapByUser(@RequestParam("userIdList") Collection<Long> userIdList);

    /**
     * 通过用户id获取用户名称、岗位名称、部门名称
     *
     * @param userIdList 用户id
     * @return MAP<用户id, 用户 、 岗位 、 部门名称>
     */
    @GetMapping(GET_NAME_BY_USER_ID)
    R<Map<Long, Map<String, Object>>> getNameByUserId(@RequestParam("userIdList") List<Long> userIdList);

    /**
     * 通过用户工号获取用户id
     *
     * @param jobNoList 用户工号
     * @return MAP<用户工号, 用户id>
     */
    @GetMapping(GET_USER_ID_BY_JOB_NO)
    R<Map<String, Long>> getUserIdByJobNo(@RequestParam("jobNoList") List<String> jobNoList);

}
